@@ -5,6 +5,7 @@ import enum
 import re
 from abc import ABC
 from collections import OrderedDict
+from copy import deepcopy
 from functools import reduce
 from decimal import Decimal, InvalidOperation
 
@@ -261,10 +262,6 @@ class String(TypedField):
         super().__set__(instance, value)
 
 
-def _foo():
-    pass
-
-
 class Function(Field):
     """
     A function. Note that this can't be any callable (it can't be a class, for example), but a real function
@@ -272,7 +269,7 @@ class Function(Field):
 
     def __set__(self, instance, value):
         def is_function(f):
-            return type(f) == type(_foo) or type(f) == type(open)
+            return type(f) == type(lambda x:x) or type(f) == type(open)
 
         def err_prefix():
             return (
@@ -366,111 +363,55 @@ class _ListStruct(list):
         super().__init__(mylist)
 
     def __setitem__(self, key, value):
-        copied = super().copy()
+        copied = self[:]
         copied.__setitem__(key, value)
-        setattr(
-            self._instance,
-            getattr(self._field_definition, "_name", None),
-            _ListStruct(
-                array=self._field_definition,
-                struct_instance=self._instance,
-                mylist=copied,
-            ),
-        )
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
 
     def append(self, value):
-        copied = super().copy()
+        copied = self[:]
         copied.append(value)
-        setattr(
-            self._instance,
-            getattr(self._field_definition, "_name", None),
-            _ListStruct(
-                array=self._field_definition,
-                struct_instance=self._instance,
-                mylist=copied,
-            ),
-        )
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
         super().append(value)
 
     def extend(self, value):
-        copied = super().copy()
+        copied = self[:]
         copied.extend(value)
         if getattr(self, "_instance", None):
             setattr(
-                self._instance,
-                getattr(self._field_definition, "_name", None),
-                _ListStruct(
-                    array=self._field_definition,
-                    struct_instance=self._instance,
-                    mylist=copied,
-                ),
+                self._instance, getattr(self._field_definition, "_name", None), copied
             )
 
     def insert(self, index: int, value):
-        copied = super().copy()
+        copied = self[:]
         copied.insert(index, value)
-        setattr(
-            self._instance,
-            getattr(self._field_definition, "_name", None),
-            _ListStruct(
-                array=self._field_definition,
-                struct_instance=self._instance,
-                mylist=copied,
-            ),
-        )
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
 
     def remove(self, ind):
-        copied = super().copy()
+        copied = self[:]
         copied.remove(ind)
-        setattr(
-            self._instance,
-            getattr(self._field_definition, "_name", None),
-            _ListStruct(
-                array=self._field_definition,
-                struct_instance=self._instance,
-                mylist=copied,
-            ),
-        )
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
 
     def pop(self, index: int = -1):
-        copied = super().copy()
+        copied = self[:]
         res = copied.pop(index)
-        setattr(
-            self._instance,
-            getattr(self._field_definition, "_name", None),
-            _ListStruct(
-                array=self._field_definition,
-                struct_instance=self._instance,
-                mylist=copied,
-            ),
-        )
-        return res
-
-    def __reduce__(self):
-        # This is a nasty hack to deal with the fact the deepcopy will call __setstate__() and after that
-        # iteratively append to the list to recreate a copy of the original list.
-        # Since __setstate__ already creates the list (used when unpickling), after the appending
-        # process we would end up with the list doubled.
-        # The root of the problem is that _ListStruct extends list, which is handled differently
-        # by deepcopy _reconstructor() function.
-        # Our hack checks if the construction function is from deepcopy. If it is - we  drop the
-        # state since deepcopy doesn't need it to create a copy.
-        res = super().__reduce__()
-        construction_func = res[0]
-        if (
-            construction_func.__module__ == "copyreg"
-            and construction_func.__name__ == "_reconstructor"
-        ):
-            return res[0], res[1]
+        setattr(self._instance, getattr(self._field_definition, "_name", None), copied)
         return res
 
     def __getstate__(self):
         return {
             "the_instance": self._instance,
             "the_array": self._field_definition,
-            "the_values": super(),
+            "the_values": self[:],
         }
 
+    def __deepcopy__(self, memo={}):
+        vals = [deepcopy(v) for v in self[:]]
+        return _ListStruct(
+            array=deepcopy(self._field_definition),
+            struct_instance=memo[id(self._instance)],
+            mylist=vals,
+        )
+
     def __setstate__(self, state):
         self._field_definition = state["the_array"]
         self._instance = state["the_instance"]
